"""
Tests for the physbiblio.webimport package.

This file is part of the physbiblio package.
"""

import physbiblio
__version__ = physbiblio.__version__

__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__all__ = ["test_webimport"]
