"""
The package that contains all the modules used by the PhysBiblio, including the `gui` subpackage.

This file is part of the physbiblio package.
"""
__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__version__ = "0.5.2"
__version_date__ = "05/07/2018"

__all__ = ["bibtexwriter", "cli", "config", "databaseCore", "database", "errors", "export", "inspireStats", "parse_accents", "pdf", "tablesDef", "tests", "view", "gui", "webimport"]
