"""
Module for retrieving bibtex information from a list of given web resources (see `__all__`).

This module is part of the physbiblio package.
"""
import physbiblio
__version__ = physbiblio.__version__

__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__all__ = ["arxiv", "doi", "inspire", "inspireoai", "isbn", "webInterf"]
